# coding=utf-8
# Licensed Materials - Property of IBM
# Copyright IBM Corp. 2019

"""
"""


__version__='1.1.0a0'

__all__ = [ 'region_match', 'FlightPathEncounter' ]
from streamsx.geospatial._geospatial import region_match, FlightPathEncounter

